<?php

namespace Drupal\academic_applications;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\webform\WebformSubmissionListBuilder;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Overrides the WebformSubmissionListBuilder.
 */
class WebFormSubmissionListBuilderOverride extends WebformSubmissionListBuilder {

  /**
   * The submission bundler.
   *
   * @var \Drupal\academic_applications\SubmissionBundler
   */
  protected $submissionBundler;

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity.manager')->getStorage($entity_type->id()),
      $container->get('academic_applications.submission_bundler')
    );
  }

  /**
   * WebFormSubmissionListBuilderOverride constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   An entity type.
   * @param \Drupal\Core\Entity\EntityStorageInterface $storage
   *   The entity storage.
   * @param \Drupal\academic_applications\SubmissionBundler $bundler
   *   The submission bundler.
   */
  public function __construct(EntityTypeInterface $entity_type, EntityStorageInterface $storage, SubmissionBundler $bundler) {
    parent::__construct($entity_type, $storage);
    $this->addUploadsColumn();
    $this->submissionBundler = $bundler;
  }

  protected function addUploadsColumn() {
    $column = [
      'title' => 'References',
      'name' => 'academic_applications_uploads',
      'format' => 'raw',
      'sort' => FALSE,
    ];
    $this->insertColumnBefore(
      'operations',
      'academic_applications_uploads',
      $column
    );
  }

  /**
   * Insert a new column before a given column.
   *
   * @param string $key
   *   The column key to insert a column before.
   * @param string $newKey
   *   The column key of the new column.
   * @param array $column
   *   The new column.
   */
  protected function insertColumnBefore($key, $newKey, $column) {
    if (array_key_exists($key, $this->columns)) {
      $new = [];
      foreach ($this->columns as $k => $value) {
        if ($k === $key) {
          $new[$newKey] = $column;
        }
        $new[$k] = $value;
      }
      $this->columns = $new;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildRowColumn(array $column, EntityInterface $entity) {
    if ($column['name'] == 'academic_applications_uploads') {
      // Add data to the custom column in each row.
      $links = [];
      $submissions = $this->submissionBundler->uploadFormSubmissions($entity);
      $count = 1;
      foreach ($submissions as $submission) {
        $link = $submission->link($count);
        $links[] = $link;
        $count++;
      }
      $row = [
        'data' => [
          '#type' => 'markup',
          '#markup' => implode(', ', $links),
        ],
      ];
    }
    else {
      $row = parent::buildRowColumn($column, $entity);
    }
    return $row;
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity) {
    $operations = parent::getDefaultOperations($entity);
    $base_route_name = $this->requestHandler->getBaseRouteName($entity, $this->sourceEntity);
    $route_parameters = $this->requestHandler->getRouteParameters($entity, $this->sourceEntity);
    if ($entity->access('view')) {
      $operations['bundle'] = [
        'title' => $this->t('Bundle'),
        'weight' => -10,
        'url' => Url::fromRoute("$base_route_name.webform_submission.bundle", $route_parameters),
      ];
    }
    return $operations;
  }

}
